/*********
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp-now-many-to-one-esp32/
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
*********/

#include <esp_now.h>
#include <WiFi.h>
#include <PZEM004Tv30.h>


#define relay_1 32
#define relay_2 33
#define relay_3 25
#define relay_4 26
#define relay_5 4
#define relay_6 0
#define relay_7 2
#define relay_8 15

#if defined(ESP32)
PZEM004Tv30 pzem(Serial2, 16, 17);
#else
PZEM004Tv30 pzem(Serial2);
#endif

// REPLACE WITH THE RECEIVER'S MAC Address
uint8_t broadcastAddress[] = { 0x68, 0xB6, 0xB3, 0x37, 0xF2, 0x74 };

// Structure example to send data
// Must match the receiver structure
typedef struct struct_message {
  int id;  // must be unique for each sender board
  int watt;
} struct_message;

typedef struct struct_command {
  int relay1;
  int relay2;
  int relay3;
  int relay4;
  int relay5;
  int relay6;
  int relay7;
  int relay8;

} struct_command;

// Create a struct_message called myData
struct_message myData;

struct_command myCommand;
// Create peer interface
esp_now_peer_info_t peerInfo;

// callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}


void OnDataRecv(const uint8_t *mac_addr, const uint8_t *incomingData, int len) {
  char macStr[18];
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",
           mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  memcpy(&myCommand, incomingData, sizeof(myCommand));



  Serial.printf("Relay_1 status: %d \n", myCommand.relay1);
  Serial.printf("Relay_2 status: %d \n", myCommand.relay2);
  Serial.printf("Relay_3 status: %d \n", myCommand.relay3);
  Serial.printf("Relay_4 status: %d \n", myCommand.relay4);
  Serial.printf("Relay_5 status: %d \n", myCommand.relay5);
  Serial.printf("Relay_6 status: %d \n", myCommand.relay6);
  Serial.printf("Relay_7 status: %d \n", myCommand.relay7);
  Serial.printf("Relay_8 status: %d \n", myCommand.relay8);

  Serial.println();
  
  if (myCommand.relay1 == 0) {
    digitalWrite(relay_1, HIGH);
  } 
  else if(myCommand.relay1 == 1) {
    digitalWrite(relay_1, LOW);
  }

  if (myCommand.relay2 == 0) {
    digitalWrite(relay_2, HIGH);
  } 
  else if(myCommand.relay2 == 1){
    digitalWrite(relay_2, LOW);
  }

  if (myCommand.relay3 == 0) {
    digitalWrite(relay_3, HIGH);
  } 
  else if(myCommand.relay3 == 1){
    digitalWrite(relay_3, LOW);
  }

  if (myCommand.relay4 == 0) {
    digitalWrite(relay_4, HIGH);
  } 
  else if(myCommand.relay4 == 1){
    digitalWrite(relay_4, LOW);
  }

  if (myCommand.relay5 == 0) {
    digitalWrite(relay_5, HIGH);
  } 
  else if(myCommand.relay5 == 1){
    digitalWrite(relay_5, LOW);
  }

  if (myCommand.relay6 == 0) {
    digitalWrite(relay_6, HIGH);
  } 
  else if(myCommand.relay6 == 1) {
    digitalWrite(relay_6, LOW);
  }

  if (myCommand.relay7 == 0) {
    digitalWrite(relay_7, HIGH);
  } 
  else if(myCommand.relay7 == 1){
    digitalWrite(relay_7, LOW);
  }

  if (myCommand.relay8 == 0) {
    digitalWrite(relay_8, HIGH);
  } 
  else if(myCommand.relay8 == 1){
    digitalWrite(relay_8, LOW);
  }
}

void setup() {
  // Init Serial Monitor
  Serial.begin(115200);


  pinMode(relay_1, OUTPUT);
  pinMode(relay_2, OUTPUT);
  pinMode(relay_3, OUTPUT);
  pinMode(relay_4, OUTPUT);
  pinMode(relay_5, OUTPUT);
  pinMode(relay_6, OUTPUT);
  pinMode(relay_7, OUTPUT);
  pinMode(relay_8, OUTPUT);

  // Set device as a Wi-Fi Station
  WiFi.mode(WIFI_STA);

  // Init ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  // Once ESPNow is successfully Init, we will register for Send CB to
  // get the status of Trasnmitted packet
  esp_now_register_send_cb(OnDataSent);

  // Register peer
  peerInfo.channel = 0;
  peerInfo.encrypt = false;


  // Add peer
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK) {
    Serial.println("Failed to add peer");
    return;
  }


  esp_now_register_recv_cb(OnDataRecv);
}

void loop() {
  Serial.print("Custom Address:");
  Serial.println(pzem.readAddress(), HEX);

  myData.id=2;

  // Read the data from the sensor
  float voltage = pzem.voltage();
  float current = pzem.current();
  float power = pzem.power();
  float energy = pzem.energy();
  float frequency = pzem.frequency();
  float pf = pzem.pf();
  // Check if the data is valid
  if (isnan(voltage)) {
    Serial.println("Error reading voltage");
  } else if (isnan(current)) {
    Serial.println("Error reading current");
  } else if (isnan(power)) {
    Serial.println("Error reading power");
  } else if (isnan(energy)) {
    Serial.println("Error reading energy");
  } else if (isnan(frequency)) {
    Serial.println("Error reading frequency");
  } else if (isnan(pf)) {
    Serial.println("Error reading power factor");
  } else {
    // Print the values to the Serial console
    Serial.print("Voltage: ");
    Serial.print(voltage);
    Serial.println("V");
    Serial.print("Current: ");
    Serial.print(current);
    Serial.println("A");
    Serial.print("Power: ");
    Serial.print(power);
    Serial.println("W");
    Serial.print("Energy: ");
    Serial.print(energy, 3);
    Serial.println("kWh");
    Serial.print("Frequency: ");
    Serial.print(frequency, 1);
    Serial.println("Hz");
    Serial.print("PF: ");
    Serial.println(pf);
    myData.watt=(int)(power*100);
  }

  // Send message via ESP-NOW
  esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *)&myData, sizeof(myData));

  if (result == ESP_OK) {
    Serial.println("Sent with success");
  } else {
    Serial.println("Error sending the data");
  }


  delay(1000);
}