/*********
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp-now-many-to-one-esp32/
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
*********/

#include <esp_now.h>
#include <WiFi.h>
#include <Wire.h>
#include <Adafruit_SI1145.h>
#include <AHTxx.h>


// REPLACE WITH THE RECEIVER'S MAC Address
uint8_t broadcastAddress[] = {0x68, 0xB6, 0xB3, 0x37, 0xF2, 0x74};

// Structure example to send data
// Must match the receiver structure


typedef struct struct_message {
  int id;  // must be unique for each sender board
  int temperature;
  int humidity;
  int uv;
  int ir;
  int vl;
} struct_message;

// Create a struct_message called myData
struct_message myData;


// Create peer interface
esp_now_peer_info_t peerInfo;


Adafruit_SI1145 uv = Adafruit_SI1145();
AHTxx aht10(AHTXX_ADDRESS_X38, AHT1x_SENSOR);

// callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}  
void setup() {
  // Init Serial Monitor
  Serial.begin(115200);


  
  // Set device as a Wi-Fi Station
  WiFi.mode(WIFI_STA);

  // Init ESP-NOW
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  // Once ESPNow is successfully Init, we will register for Send CB to
  // get the status of Trasnmitted packet
  esp_now_register_send_cb(OnDataSent);
  
  // Register peer
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;
  
  
  // Add peer        
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }

  Wire.begin(48,47);
  Serial.println();
  while (aht10.begin() != true) //for ESP-01 use aht10.begin(0, 2);
  {
    Serial.println(F("AHT1x not connected or fail to load calibration coefficient")); //(F()) save string to flash & keeps dynamic memory free

    delay(5000);
  }
  Serial.println(F("AHT10 OK"));
  while (!uv.begin());
}
 
void loop() {

 // Set values to send
  Serial.print("Vis: ");
  Serial.print(uv.readVisible());
  Serial.print("mW/cm^2");
  Serial.print("     IR: ");
  Serial.print(uv.readIR());
  Serial.print("mW/cm^2");
  float UVintensity = uv.readUV();
  // the intensity is multiplied by 100 so to get the
  // integer intensity, divide by 100!
  int UV = (int)(UVintensity*10.0);
  Serial.print("     UV: ");
  Serial.print(UVintensity);
  Serial.println(F("mW/cm^2"));

  myData.id=3;
  myData.temperature=(int)(aht10.readTemperature()*100.0);
  myData.humidity=(int)(aht10.readHumidity()*100.0);
  myData.uv=UV;
  myData.ir=uv.readIR();
  myData.vl=uv.readVisible();

  

  Serial.print("Temp ");
  Serial.println(myData.temperature);
  Serial.print("Humidity ");
  Serial.println(myData.humidity);

  

  

  // Send message via ESP-NOW
  esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &myData, sizeof(myData));
   
  if (result == ESP_OK) {
    Serial.println("Sent with success");
  }
  else {
    Serial.println("Error sending the data");
  }

  
  delay(1000);
}